//
//  test_GVF.cpp
//  Unit tests for GFV
//
//  Created by Francisco Bernardo on 17/02/2017.
//  Copyright © 2017 Goldsmiths. All rights reserved.
//

#define CATCH_CONFIG_MAIN  // This tells Catch to provide a main() - only do this in one cpp file
#include "catch.hpp"
#include "machineLearning.h"
#include "trainingData.h"


TEST_CASE( "Tests default GVF ctor.", "[GVF]" ) {

    GVF * gvf = new GVF();
    gvf->translate(false);
    gvf->segmentation(false);
    
    REQUIRE(gvf != NULL);

}

TEST_CASE( "Tests default GVFGesture ctor.", "[GVF]" ) {
    
    GVF * gvf = new GVF();
    GVFGesture gesture;
    GVFOutcomes outcomes;
    
    gvf->translate(false);
    gvf->segmentation(false);
    
    gvf->setNumberOfParticles(1000);
    gvf->setTolerance(0.2f);
    gvf->setScalingsVariance(0.00001f); //0.00001f);   //0.000002f);   //0.00004f);
    gvf->setDynamicsVariance(0.0001f);
   
    vector<vector<float>> trainingData;
    
    if(gvf->getState() == GVF::STATE_LEARNING)
    {
        gvf->startGesture();
        
        for (vector<vector<float>>::iterator frame = trainingData.begin(); frame != trainingData.end(); frame++)
        {
            // Fill the template
            gesture.addObservation(*frame);
        }
    }
    
    gvf->setState(GVF::STATE_FOLLOWING);
    
    gesture.clear();
    trainingData.clear();
    
    vector<vector<float>> testData;
    
    if (gvf->getState()==GVF::STATE_FOLLOWING)
    {
        for (vector<vector<float>>::iterator frame = testData.begin(); frame != testData.end(); frame++)
        {
            // Fill the template
            gesture.addObservation(*frame);
        }
        
        gvf->update(gesture.getLastObservation());
    }
    
        
//    float phase = gvf->getOutcomes().estimations[0].alignment;
//    float speed = gvf->getOutcomes().estimations[0].dynamics[0];
//    
//    getDynamicsVariance();
//    getScalingsVariance();
//    getRotationsVariance();
    
    REQUIRE( gvf != NULL);
    
}

SCENARIO("Test GVF Regression", "[machineLearning]")
{
    GIVEN("gvf object and training dataset")
    {
        rapidmix::gvfTemporalVariation gvf;
        rapidmix::trainingData myData;
        
        std::vector<double> input = { 0.2, 0.7 };
        std::vector<double> output = { 3.0 };
        myData.addElement(input, output);
        
        input = { 2.0, 44.2 };
        output = { 20.14 };
        myData.addElement(input, output);
        
        gvf.train(myData);
        
        gvf.process(input);
        
        
        std::string filepath2 = "/var/tmp/modelSetDescription_gvf.json";
//        gvf.writeJSON(filepath2);
        
        
        std::vector<double> inputVec = { 2.0, 44.2 };
        
        WHEN("when gvf model is read from file")
        {
            THEN("run models and compare")
            {
//                REQUIRE(gvf.run(inputVec)[0] == gvfFromFile.run(inputVec)[0]);
            }
        }
        
        WHEN("when gvf model is read from JSON stream")
        {
            THEN("run models and compare")
            {
//                REQUIRE(gvf.run(inputVec)[0] == gvfFromString.run(inputVec)[0]);
            }
        }
    }
}

        
/*
unsigned int runGVF(){

    
    GVFGesture       currentGesture;
    GVFConfig        config;
    GVFOutcomes      outcomes;
    
    
    
    // START UNIT TEST LOOP HERE
    
    
    // Init GVF
    config.inputDimensions  = 8;
    config.translate        = false;
    config.segmentation     = false;
    
    GVF * gvf = new ofxGVF(config);
    
    // change set parameters (from default)
    gvf->setScalingsVariance(0.00001f); //0.00001f);   //0.000002f);   //0.00004f);
    gvf->setDynamicsVariance(0.0001f);
    
    
    
    std::vector<std::string> trainingFiles;
    
    // Data structure for training data
    vector<vector<vector<float>>> trainingData = readData(trainPath, participantRegExp, trainingFiles);
    
    for (vector<vector<vector<float>>>::iterator trainingGesture = trainingData.begin(); trainingGesture != trainingData.end(); trainingGesture++)
    {
        currentGesture.clear();
        for (vector<vector<float>>::iterator frame = (*trainingGesture).begin(); frame != (*trainingGesture).end(); frame++)
        {
            // Fill the template
            currentGesture.addObservation(*frame);
        }
        // Add the template at the end of the gesture
        gvf->addGestureTemplate(currentGesture);
    }
    
    
    // Set Following Mode (fitting here)
    gvf->setState(ofxGVF::STATE_FOLLOWING);
    
    
    std::vector<std::string> testFilesList;
    
    vector<GVFOutput> gvfOutputs = readTestData(testPath, participantRegExp, outputPath);
    
    float tempAlignmentDistance = 0.0;
    
    float alignmentDistance;
    
    for (vector<GVFOutput>::iterator gvfOutput = gvfOutputs.begin(); gvfOutput != gvfOutputs.end(); gvfOutput++)
    {
        
        // here you restart before testing with an indivudal testing gesture
        gvf->restart();
        
//        boost::filesystem::ofstream outfile(gvfOutput->fileOut);
        
        for (vector<vector<float>>::iterator frame = (gvfOutput->testData).begin(); frame != (gvfOutput->testData).end(); frame++) {
            
            currentGesture.addObservation(*frame);
            
            gvf->update(currentGesture.getLastObservation());
            
            ofxGVFOutcomes outcomes = gvf->getOutcomes();
            
            if (outcomes.most_probable >= 0){
                
                outfile << outcomes.estimations[0].scalings[0] << ' ' << outcomes.estimations[0].scalings[1] << ' ' << outcomes.estimations[0].scalings[2] << ' ' << outcomes.estimations[0].scalings[3] << ' ' << outcomes.estimations[0].scalings[4] << ' ' << outcomes.estimations[0].scalings[5] << ' ' << outcomes.estimations[0].scalings[6] << ' ' << outcomes.estimations[0].scalings[7] << ' ' << outcomes.estimations[1].scalings[0] << ' ' << outcomes.estimations[1].scalings[1] << ' ' << outcomes.estimations[1].scalings[2] << ' ' << outcomes.estimations[1].scalings[3] << ' ' << outcomes.estimations[1].scalings[4] << ' ' << outcomes.estimations[1].scalings[5] << ' ' << outcomes.estimations[1].scalings[6] << ' ' << outcomes.estimations[1].scalings[7] << ' ' << outcomes.estimations[2].scalings[0] << ' ' << outcomes.estimations[2].scalings[1] << ' ' << outcomes.estimations[2].scalings[2] << ' ' << outcomes.estimations[2].scalings[3] << ' ' << outcomes.estimations[2].scalings[4] << ' ' << outcomes.estimations[2].scalings[5] << ' ' << outcomes.estimations[2].scalings[6] << ' ' << outcomes.estimations[2].scalings[7] << ' ' << outcomes.estimations[0].dynamics[0] << ' ' << outcomes.estimations[0].dynamics[1] << ' ' << outcomes.estimations[1].dynamics[0] << ' ' << outcomes.estimations[1].dynamics[1] << ' ' << outcomes.estimations[2].dynamics[0] << ' ' << outcomes.estimations[2].dynamics[1] << ' ' << outcomes.estimations[0].alignment << ' ' << outcomes.estimations[1].alignment << ' ' << outcomes.estimations[2].alignment << ' ' << '0' << ' ' << outcomes.estimations[0].likelihood << ' ' << outcomes.estimations[1].likelihood << ' ' << outcomes.estimations[2].likelihood << ' ' << '1' << ' ' << '0' << ' ' << '0' << std::endl;
            }
            else
            {
                outfile << "Error" << endl;
            }
            
            // reconstruction and distance computation
            int alignmentindex = 0;
            
            if (gvf->getMostProbableGestureIndex()>=0)
                alignmentindex = trainingData[gvf->getMostProbableGestureIndex()].size() * outcomes.estimations[gvf->getMostProbableGestureIndex()].alignment;
            
            vector<float> alignedFrame = trainingData[gvf->getMostProbableGestureIndex()][alignmentindex];
            float distanceRefAligned = 0.0;
            
            for (int i=0; i<alignedFrame.size(); i++){
                alignedFrame[i]*=outcomes.estimations[gvf->getMostProbableGestureIndex()].scalings[i];
                distanceRefAligned+=pow(alignedFrame[i]-(*frame)[i],2);
            }
            
            tempAlignmentDistance += sqrt(distanceRefAligned);
        }
        
        alignmentDistance = tempAlignmentDistance / (float)(gvfOutput->testData).size();
        
        outfile.close();
    }
    return 0;
}

*/


//TEST_CASE( "Factorials are computed", "[factorial]" ) {
//    REQUIRE( Factorial(1) == 1 );
//    REQUIRE( Factorial(2) == 2 );
//    REQUIRE( Factorial(3) == 6 );
//    REQUIRE( Factorial(10) == 3628800 );
//}
